/* 
   Unix SMB/Netbios implementation.
   Version 3.0
   client transaction calls
   Copyright (C) Andrew Tridgell 1994-1998
   Copyright (C) Luke Kenneth Casson Leighton 1996-1999
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/* MKPROTO: lib_smb_proto.h		(do not remove this line) */

#include "includes.h"
#include "libsamba.h"
#include "tng_misc.h"
#include "byteorder.h"
#include "libsmb.h"


/****************************************************************************
  send a SMB trans or trans2 request
  ****************************************************************************/
BOOL cli_send_trans(struct cli_state *cli, int trans, 
		    const char *pipe_name, 
                           int fid, int flags,
                           uint16 *setup, size_t lsetup, size_t msetup,
                           char *param, size_t lparam, size_t mparam,
                           char *data, size_t ldata, size_t mdata)
{
	int i;
	size_t this_ldata, this_lparam;
	size_t tot_data=0, tot_param=0;
	char *outdata,*outparam;
	char *p;
	int pipe_name_len=0;

	this_lparam = MIN(lparam,cli->max_xmit - (500+lsetup*2)); /* hack */
	this_ldata = MIN(ldata,cli->max_xmit - (500+lsetup*2+this_lparam));

	memset(cli->outbuf,'\0',smb_size);
	set_message(cli->outbuf,14+lsetup,0,True);
	CVAL(cli->outbuf,smb_com) = trans;
	SSVAL(cli->outbuf,smb_tid, cli->cnum);
	cli_setup_packet(cli);

	if (pipe_name) {
		pipe_name_len = clistr_push(cli, smb_buf(cli->outbuf), pipe_name, -1, STR_TERMINATE|STR_ALIGN);
	}

	outparam = smb_buf(cli->outbuf)+(trans==SMBtrans ? pipe_name_len : 3);
	outdata = outparam+this_lparam;

	/* primary request */
	SSVAL(cli->outbuf,smb_tpscnt,lparam);	/* tpscnt */
	SSVAL(cli->outbuf,smb_tdscnt,ldata);	/* tdscnt */
	SSVAL(cli->outbuf,smb_mprcnt,mparam);	/* mprcnt */
	SSVAL(cli->outbuf,smb_mdrcnt,mdata);	/* mdrcnt */
	SCVAL(cli->outbuf,smb_msrcnt,msetup);	/* msrcnt */
	SSVAL(cli->outbuf,smb_flags,flags);	/* flags */
	SIVAL(cli->outbuf,smb_timeout,0);		/* timeout */
	SSVAL(cli->outbuf,smb_pscnt,this_lparam);	/* pscnt */
	SSVAL(cli->outbuf,smb_psoff,smb_offset(outparam,cli->outbuf)); /* psoff */
	SSVAL(cli->outbuf,smb_dscnt,this_ldata);	/* dscnt */
	SSVAL(cli->outbuf,smb_dsoff,smb_offset(outdata,cli->outbuf)); /* dsoff */
	SCVAL(cli->outbuf,smb_suwcnt,lsetup);	/* suwcnt */
	for (i=0;i<lsetup;i++)		/* setup[] */
		SSVAL(cli->outbuf,smb_setup+i*2,setup[i]);
	p = smb_buf(cli->outbuf);
	if (trans != SMBtrans) {
		*p++ = 0;  /* put in a null smb_name */
		*p++ = 'D'; *p++ = ' ';	/* observed in OS/2 */
	}
	if (this_lparam)			/* param[] */
		memcpy(outparam,param,this_lparam);
	if (this_ldata)			/* data[] */
		memcpy(outdata,data,this_ldata);
	cli_setup_bcc(cli, outdata+this_ldata);

	cli_send_smb(cli);

	if (this_ldata < ldata || this_lparam < lparam) {
		/* receive interim response */
		if (!cli_receive_smb(cli) || 
		    CVAL(cli->inbuf,smb_rcls) != 0) {
			return(False);
		}      

		tot_data = this_ldata;
		tot_param = this_lparam;
		
		while (tot_data < ldata || tot_param < lparam)  {
			this_lparam = MIN(lparam-tot_param,cli->max_xmit - 500); /* hack */
			this_ldata = MIN(ldata-tot_data,cli->max_xmit - (500+this_lparam));

			set_message(cli->outbuf,trans==SMBtrans?8:9,0,True);
			CVAL(cli->outbuf,smb_com) = trans==SMBtrans ? SMBtranss : SMBtranss2;
			
			outparam = smb_buf(cli->outbuf);
			outdata = outparam+this_lparam;
			
			/* secondary request */
			SSVAL(cli->outbuf,smb_tpscnt,lparam);	/* tpscnt */
			SSVAL(cli->outbuf,smb_tdscnt,ldata);	/* tdscnt */
			SSVAL(cli->outbuf,smb_spscnt,this_lparam);	/* pscnt */
			SSVAL(cli->outbuf,smb_spsoff,smb_offset(outparam,cli->outbuf)); /* psoff */
			SSVAL(cli->outbuf,smb_spsdisp,tot_param);	/* psdisp */
			SSVAL(cli->outbuf,smb_sdscnt,this_ldata);	/* dscnt */
			SSVAL(cli->outbuf,smb_sdsoff,smb_offset(outdata,cli->outbuf)); /* dsoff */
			SSVAL(cli->outbuf,smb_sdsdisp,tot_data);	/* dsdisp */
			if (trans==SMBtrans2)
				SSVALS(cli->outbuf,smb_sfid,fid);		/* fid */
			if (this_lparam)			/* param[] */
				memcpy(outparam,param+tot_param,this_lparam);
			if (this_ldata)			/* data[] */
				memcpy(outdata,data+tot_data,this_ldata);
			cli_setup_bcc(cli, outdata+this_ldata);
			
			cli_send_smb(cli);
			
			tot_data += this_ldata;
			tot_param += this_lparam;
		}
	}

	return(True);
}


/****************************************************************************
  receive a SMB trans or trans2 response allocating the necessary memory
  ****************************************************************************/
BOOL cli_receive_trans(struct cli_state *cli,int trans,
                              char **param, size_t *param_len,
                              char **data, size_t *data_len)
{
	size_t total_data=0;
	size_t total_param=0;
	size_t this_data, this_param;
	
	*data_len = *param_len = 0;

	if (!cli_receive_smb(cli))
		return False;

	/* sanity check */
	if (CVAL(cli->inbuf,smb_com) != trans) {
		DEBUG(0,("Expected %s response, got command 0x%02x\n",
			 trans==SMBtrans?"SMBtrans":"SMBtrans2", 
			 CVAL(cli->inbuf,smb_com)));
		return(False);
	}

	if (cli_dos_error(cli, NULL, NULL))
	{
		return False;
	}

	/* parse out the lengths */
	total_data = SVAL(cli->inbuf,smb_tdrcnt);
	total_param = SVAL(cli->inbuf,smb_tprcnt);

	/* allocate it */
	if (total_data!=0) {
		if (!g_xrenew(char, data, total_data)) {
			DEBUG(0,("cli_receive_trans: failed to enlarge data buffer\n"));
			return False;
		}
	}

	if (total_param!=0) {
		if (!g_xrenew(char, param, total_param)) {
			DEBUG(0,("cli_receive_trans: failed to enlarge param buffer\n"));
			return False;
		}
	}

	while (1)  {
		this_data = SVAL(cli->inbuf,smb_drcnt);
		this_param = SVAL(cli->inbuf,smb_prcnt);

		if (this_data)
		{
			size_t off_out = SVAL(cli->inbuf, smb_drdisp);
			size_t off_in = SVAL(cli->inbuf, smb_droff);

			if (SIZE_OVERFLOW3(this_data, *data_len, total_data) ||
			    SIZE_OVERFLOW3(this_data, off_out, total_data) ||
			    SIZE_OVERFLOW3(this_data, off_in, cli->bufsize))
			{
				DEBUG(1, ("Data overflow in cli_receive_trans\n"));
				return False;
			}
			memcpy(*data + off_out, smb_base(cli->inbuf) + off_in,
			       this_data);
		}
		if (this_param)
		{
			size_t off_out = SVAL(cli->inbuf, smb_prdisp);
			size_t off_in = SVAL(cli->inbuf, smb_proff);

			if (SIZE_OVERFLOW3(this_param, *param_len, total_param) ||
			    SIZE_OVERFLOW3(off_out, this_param, total_param) ||
			    SIZE_OVERFLOW3(off_in, this_param, cli->bufsize))
			{
				DEBUG(1,("Param overflow in cli_receive_trans\n"));
				return False;
			}
			memcpy(*param + off_out, smb_base(cli->inbuf) + off_in,
			       this_param);
		}
		*data_len += this_data;
		*param_len += this_param;

		if (total_data <= *data_len && total_param <= *param_len)
			break;
		
		if (!cli_receive_smb(cli))
			return False;

		/* sanity check */
		if (CVAL(cli->inbuf,smb_com) != trans) {
			DEBUG(0,("Expected %s response, got command 0x%02x\n",
				 trans==SMBtrans?"SMBtrans":"SMBtrans2", 
				 CVAL(cli->inbuf,smb_com)));
			return(False);
		}

		if (cli_dos_error(cli, NULL, NULL))
		{
			return(False);
		}

		/* parse out the total lengths again - they can shrink! */
		if (SVAL(cli->inbuf, smb_tdrcnt) < total_data)
			total_data = SVAL(cli->inbuf, smb_tdrcnt);
		if (SVAL(cli->inbuf, smb_tprcnt) < total_param)
			total_param = SVAL(cli->inbuf, smb_tprcnt);
 
		if (total_data <= *data_len && total_param <= *param_len)
			break;
 
	}
	
	return(True);
}




/****************************************************************************
  send a SMB nttrans request
  ****************************************************************************/



/****************************************************************************
  receive a SMB nttrans response allocating the necessary memory
  ****************************************************************************/
