/* NT error codes.  please read nterr.h */

#include "includes.h"
#include "libsamba.h"
#include "libsmb.h"
#include "nterr.h"
#include "doserr.h"
#include "winerror.h"

typedef struct
{
	const char *nt_errstr;
	NTSTATUS nt_errcode;

} nt_err_code_struct;

static const nt_err_code_struct nt_errs[] =
{
	{ "SUCCESS", NT_STATUS_NOPROBLEMO },
#define X(a) { #a, a },
	X(STATUS_SOME_NOT_MAPPED)
	X(STATUS_UNSUCCESSFUL)
	X(STATUS_NOT_IMPLEMENTED)
	X(STATUS_INVALID_INFO_CLASS)
	X(STATUS_INFO_LENGTH_MISMATCH)
	X(STATUS_ACCESS_VIOLATION)
	X(STATUS_BUFFER_OVERFLOW)
	X(STATUS_IN_PAGE_ERROR)
	X(STATUS_PAGEFILE_QUOTA)
	X(STATUS_INVALID_HANDLE)
	X(STATUS_BAD_INITIAL_STACK)
	X(STATUS_BAD_INITIAL_PC)
	X(STATUS_INVALID_CID)
	X(STATUS_TIMER_NOT_CANCELED)
	X(STATUS_INVALID_PARAMETER)
	X(STATUS_NO_SUCH_DEVICE)
	X(STATUS_NO_SUCH_FILE)
	X(STATUS_INVALID_DEVICE_REQUEST)
	X(STATUS_END_OF_FILE)
	X(STATUS_WRONG_VOLUME)
	X(STATUS_NO_MEDIA_IN_DEVICE)
	X(STATUS_UNRECOGNIZED_MEDIA)
	X(STATUS_NONEXISTENT_SECTOR)
	X(STATUS_MORE_PROCESSING_REQUIRED)
	X(STATUS_NO_MEMORY)
	X(STATUS_CONFLICTING_ADDRESSES)
	X(STATUS_NOT_MAPPED_VIEW)
	X(STATUS_UNABLE_TO_FREE_VM)
	X(STATUS_UNABLE_TO_DELETE_SECTION)
	X(STATUS_INVALID_SYSTEM_SERVICE)
	X(STATUS_ILLEGAL_INSTRUCTION)
	X(STATUS_INVALID_LOCK_SEQUENCE)
	X(STATUS_INVALID_VIEW_SIZE)
	X(STATUS_INVALID_FILE_FOR_SECTION)
	X(STATUS_ALREADY_COMMITTED)
	X(STATUS_ACCESS_DENIED)
	X(STATUS_BUFFER_TOO_SMALL)
	X(STATUS_OBJECT_TYPE_MISMATCH)
	X(STATUS_NONCONTINUABLE_EXCEPTION)
	X(STATUS_INVALID_DISPOSITION)
	X(STATUS_UNWIND)
	X(STATUS_BAD_STACK)
	X(STATUS_INVALID_UNWIND_TARGET)
	X(STATUS_NOT_LOCKED)
	X(STATUS_PARITY_ERROR)
	X(STATUS_UNABLE_TO_DECOMMIT_VM)
	X(STATUS_NOT_COMMITTED)
	X(STATUS_INVALID_PORT_ATTRIBUTES)
	X(STATUS_PORT_MESSAGE_TOO_LONG)
	X(STATUS_INVALID_PARAMETER_MIX)
	X(STATUS_INVALID_QUOTA_LOWER)
	X(STATUS_DISK_CORRUPT_ERROR)
	X(STATUS_OBJECT_NAME_INVALID)
	X(STATUS_OBJECT_NAME_NOT_FOUND)
	X(STATUS_OBJECT_NAME_COLLISION)
	X(NT_STATUS_HANDLE_NOT_WAITABLE)
	X(STATUS_PORT_DISCONNECTED)
	X(STATUS_DEVICE_ALREADY_ATTACHED)
	X(STATUS_OBJECT_PATH_INVALID)
	X(STATUS_OBJECT_PATH_NOT_FOUND)
	X(STATUS_OBJECT_PATH_SYNTAX_BAD)
	X(STATUS_DATA_OVERRUN)
	X(STATUS_DATA_LATE_ERROR)
	X(STATUS_DATA_ERROR)
	X(STATUS_CRC_ERROR)
	X(STATUS_SECTION_TOO_BIG)
	X(STATUS_PORT_CONNECTION_REFUSED)
	X(STATUS_INVALID_PORT_HANDLE)
	X(STATUS_SHARING_VIOLATION)
	X(STATUS_QUOTA_EXCEEDED)
	X(STATUS_INVALID_PAGE_PROTECTION)
	X(STATUS_MUTANT_NOT_OWNED)
	X(STATUS_SEMAPHORE_LIMIT_EXCEEDED)
	X(STATUS_PORT_ALREADY_SET)
	X(STATUS_SECTION_NOT_IMAGE)
	X(STATUS_SUSPEND_COUNT_EXCEEDED)
	X(STATUS_THREAD_IS_TERMINATING)
	X(STATUS_BAD_WORKING_SET_LIMIT)
	X(STATUS_INCOMPATIBLE_FILE_MAP)
	X(STATUS_SECTION_PROTECTION)
	X(STATUS_EAS_NOT_SUPPORTED)
	X(STATUS_EA_TOO_LARGE)
	X(STATUS_NONEXISTENT_EA_ENTRY)
	X(STATUS_NO_EAS_ON_FILE)
	X(STATUS_EA_CORRUPT_ERROR)
	X(STATUS_FILE_LOCK_CONFLICT)
	X(STATUS_LOCK_NOT_GRANTED)
	X(STATUS_DELETE_PENDING)
	X(STATUS_CTL_FILE_NOT_SUPPORTED)
	X(STATUS_UNKNOWN_REVISION)
	X(STATUS_REVISION_MISMATCH)
	X(STATUS_INVALID_OWNER)
	X(STATUS_INVALID_PRIMARY_GROUP)
	X(STATUS_NO_IMPERSONATION_TOKEN)
	X(STATUS_CANT_DISABLE_MANDATORY)
	X(STATUS_NO_LOGON_SERVERS)
	X(STATUS_NO_SUCH_LOGON_SESSION)
	X(STATUS_NO_SUCH_PRIVILEGE)
	X(STATUS_PRIVILEGE_NOT_HELD)
	X(STATUS_INVALID_ACCOUNT_NAME)
	X(STATUS_USER_EXISTS)
	X(STATUS_NO_SUCH_USER)
	X(STATUS_GROUP_EXISTS)
	X(STATUS_NO_SUCH_GROUP)
	X(STATUS_MEMBER_IN_GROUP)
	X(STATUS_MEMBER_NOT_IN_GROUP)
	X(STATUS_LAST_ADMIN)
	X(STATUS_WRONG_PASSWORD)
	X(STATUS_ILL_FORMED_PASSWORD)
	X(STATUS_PASSWORD_RESTRICTION)
	X(STATUS_LOGON_FAILURE)
	X(STATUS_ACCOUNT_RESTRICTION)
	X(STATUS_INVALID_LOGON_HOURS)
	X(STATUS_INVALID_WORKSTATION)
	X(STATUS_PASSWORD_EXPIRED)
	X(STATUS_ACCOUNT_DISABLED)
	X(STATUS_NONE_MAPPED)
	X(STATUS_TOO_MANY_LUIDS_REQUESTED)
	X(STATUS_LUIDS_EXHAUSTED)
	X(STATUS_INVALID_SUB_AUTHORITY)
	X(STATUS_INVALID_ACL)
	X(STATUS_INVALID_SID)
	X(STATUS_INVALID_SECURITY_DESCR)
	X(STATUS_PROCEDURE_NOT_FOUND)
	X(STATUS_INVALID_IMAGE_FORMAT)
	X(STATUS_NO_TOKEN)
	X(STATUS_BAD_INHERITANCE_ACL)
	X(STATUS_RANGE_NOT_LOCKED)
	X(STATUS_DISK_FULL)
	X(STATUS_SERVER_DISABLED)
	X(STATUS_SERVER_NOT_DISABLED)
	X(STATUS_TOO_MANY_GUIDS_REQUESTED)
	X(STATUS_GUIDS_EXHAUSTED)
	X(STATUS_INVALID_ID_AUTHORITY)
	X(STATUS_AGENTS_EXHAUSTED)
	X(STATUS_INVALID_VOLUME_LABEL)
	X(STATUS_SECTION_NOT_EXTENDED)
	X(STATUS_NOT_MAPPED_DATA)
	X(STATUS_RESOURCE_DATA_NOT_FOUND)
	X(STATUS_RESOURCE_TYPE_NOT_FOUND)
	X(STATUS_RESOURCE_NAME_NOT_FOUND)
	X(STATUS_ARRAY_BOUNDS_EXCEEDED)
	X(STATUS_FLOAT_DENORMAL_OPERAND)
	X(STATUS_FLOAT_DIVIDE_BY_ZERO)
	X(STATUS_FLOAT_INEXACT_RESULT)
	X(STATUS_FLOAT_INVALID_OPERATION)
	X(STATUS_FLOAT_OVERFLOW)
	X(STATUS_FLOAT_STACK_CHECK)
	X(STATUS_FLOAT_UNDERFLOW)
	X(STATUS_INTEGER_DIVIDE_BY_ZERO)
	X(STATUS_INTEGER_OVERFLOW)
	X(STATUS_PRIVILEGED_INSTRUCTION)
	X(STATUS_TOO_MANY_PAGING_FILES)
	X(STATUS_FILE_INVALID)
	X(STATUS_ALLOTTED_SPACE_EXCEEDED)
	X(STATUS_INSUFFICIENT_RESOURCES)
	X(STATUS_DFS_EXIT_PATH_FOUND)
	X(STATUS_DEVICE_DATA_ERROR)
	X(STATUS_DEVICE_NOT_CONNECTED)
	X(STATUS_DEVICE_POWER_FAILURE)
	X(STATUS_FREE_VM_NOT_AT_BASE)
	X(STATUS_MEMORY_NOT_ALLOCATED)
	X(STATUS_WORKING_SET_QUOTA)
	X(STATUS_MEDIA_WRITE_PROTECTED)
	X(STATUS_DEVICE_NOT_READY)
	X(STATUS_INVALID_GROUP_ATTRIBUTES)
	X(STATUS_BAD_IMPERSONATION_LEVEL)
	X(STATUS_CANT_OPEN_ANONYMOUS)
	X(STATUS_BAD_VALIDATION_CLASS)
	X(STATUS_BAD_TOKEN_TYPE)
	X(STATUS_BAD_MASTER_BOOT_RECORD)
	X(STATUS_INSTRUCTION_MISALIGNMENT)
	X(STATUS_INSTANCE_NOT_AVAILABLE)
	X(STATUS_PIPE_NOT_AVAILABLE)
	X(STATUS_INVALID_PIPE_STATE)
	X(STATUS_PIPE_BUSY)
	X(STATUS_ILLEGAL_FUNCTION)
	X(STATUS_PIPE_DISCONNECTED)
	X(STATUS_PIPE_CLOSING)
	X(STATUS_PIPE_CONNECTED)
	X(STATUS_PIPE_LISTENING)
	X(STATUS_INVALID_READ_MODE)
	X(STATUS_IO_TIMEOUT)
	X(STATUS_FILE_FORCED_CLOSED)
	X(STATUS_PROFILING_NOT_STARTED)
	X(STATUS_PROFILING_NOT_STOPPED)
	X(STATUS_COULD_NOT_INTERPRET)
	X(STATUS_FILE_IS_A_DIRECTORY)
	X(STATUS_NOT_SUPPORTED)
	X(STATUS_REMOTE_NOT_LISTENING)
	X(STATUS_DUPLICATE_NAME)
	X(STATUS_BAD_NETWORK_PATH)
	X(STATUS_NETWORK_BUSY)
	X(STATUS_DEVICE_DOES_NOT_EXIST)
	X(STATUS_TOO_MANY_COMMANDS)
	X(STATUS_ADAPTER_HARDWARE_ERROR)
	X(STATUS_INVALID_NETWORK_RESPONSE)
	X(STATUS_UNEXPECTED_NETWORK_ERROR)
	X(STATUS_BAD_REMOTE_ADAPTER)
	X(STATUS_PRINT_QUEUE_FULL)
	X(STATUS_NO_SPOOL_SPACE)
	X(STATUS_PRINT_CANCELLED)
	X(STATUS_NETWORK_NAME_DELETED)
	X(STATUS_NETWORK_ACCESS_DENIED)
	X(STATUS_BAD_DEVICE_TYPE)
	X(STATUS_BAD_NETWORK_NAME)
	X(STATUS_TOO_MANY_NAMES)
	X(STATUS_TOO_MANY_SESSIONS)
	X(STATUS_SHARING_PAUSED)
	X(STATUS_REQUEST_NOT_ACCEPTED)
	X(STATUS_REDIRECTOR_PAUSED)
	X(STATUS_NET_WRITE_FAULT)
	X(STATUS_PROFILING_AT_LIMIT)
	X(STATUS_NOT_SAME_DEVICE)
	X(STATUS_FILE_RENAMED)
	X(STATUS_VIRTUAL_CIRCUIT_CLOSED)
	X(STATUS_NO_SECURITY_ON_OBJECT)
	X(STATUS_CANT_WAIT)
	X(STATUS_PIPE_EMPTY)
	X(STATUS_CANT_ACCESS_DOMAIN_INFO)
	X(STATUS_CANT_TERMINATE_SELF)
	X(STATUS_INVALID_SERVER_STATE)
	X(STATUS_INVALID_DOMAIN_STATE)
	X(STATUS_INVALID_DOMAIN_ROLE)
	X(STATUS_NO_SUCH_DOMAIN)
	X(STATUS_DOMAIN_EXISTS)
	X(STATUS_DOMAIN_LIMIT_EXCEEDED)
	X(STATUS_OPLOCK_NOT_GRANTED)
	X(STATUS_INVALID_OPLOCK_PROTOCOL)
	X(STATUS_INTERNAL_DB_CORRUPTION)
	X(STATUS_INTERNAL_ERROR)
	X(STATUS_GENERIC_NOT_MAPPED)
	X(STATUS_BAD_DESCRIPTOR_FORMAT)
	X(STATUS_INVALID_USER_BUFFER)
	X(STATUS_UNEXPECTED_IO_ERROR)
	X(STATUS_UNEXPECTED_MM_CREATE_ERR)
	X(STATUS_UNEXPECTED_MM_MAP_ERROR)
	X(STATUS_UNEXPECTED_MM_EXTEND_ERR)
	X(STATUS_NOT_LOGON_PROCESS)
	X(STATUS_LOGON_SESSION_EXISTS)
	X(STATUS_INVALID_PARAMETER_1)
	X(STATUS_INVALID_PARAMETER_2)
	X(STATUS_INVALID_PARAMETER_3)
	X(STATUS_INVALID_PARAMETER_4)
	X(STATUS_INVALID_PARAMETER_5)
	X(STATUS_INVALID_PARAMETER_6)
	X(STATUS_INVALID_PARAMETER_7)
	X(STATUS_INVALID_PARAMETER_8)
	X(STATUS_INVALID_PARAMETER_9)
	X(STATUS_INVALID_PARAMETER_10)
	X(STATUS_INVALID_PARAMETER_11)
	X(STATUS_INVALID_PARAMETER_12)
	X(STATUS_REDIRECTOR_NOT_STARTED)
	X(STATUS_REDIRECTOR_STARTED)
	X(STATUS_STACK_OVERFLOW)
	X(STATUS_NO_SUCH_PACKAGE)
	X(STATUS_BAD_FUNCTION_TABLE)
	X(STATUS_DIRECTORY_NOT_EMPTY)
	X(STATUS_FILE_CORRUPT_ERROR)
	X(STATUS_NOT_A_DIRECTORY)
	X(STATUS_BAD_LOGON_SESSION_STATE)
	X(STATUS_LOGON_SESSION_COLLISION)
	X(STATUS_NAME_TOO_LONG)
	X(STATUS_FILES_OPEN)
	X(STATUS_CONNECTION_IN_USE)
	X(STATUS_MESSAGE_NOT_FOUND)
	X(STATUS_PROCESS_IS_TERMINATING)
	X(STATUS_INVALID_LOGON_TYPE)
	X(STATUS_NO_GUID_TRANSLATION)
	X(STATUS_CANNOT_IMPERSONATE)
	X(STATUS_IMAGE_ALREADY_LOADED)
	X(STATUS_ABIOS_NOT_PRESENT)
	X(STATUS_ABIOS_LID_NOT_EXIST)
	X(STATUS_ABIOS_LID_ALREADY_OWNED)
	X(STATUS_ABIOS_NOT_LID_OWNER)
	X(STATUS_ABIOS_INVALID_COMMAND)
	X(STATUS_ABIOS_INVALID_LID)
	X(STATUS_ABIOS_SELECTOR_NOT_AVAILABLE)
	X(STATUS_ABIOS_INVALID_SELECTOR)
	X(STATUS_NO_LDT)
	X(STATUS_INVALID_LDT_SIZE)
	X(STATUS_INVALID_LDT_OFFSET)
	X(STATUS_INVALID_LDT_DESCRIPTOR)
	X(STATUS_INVALID_IMAGE_NE_FORMAT)
	X(STATUS_RXACT_INVALID_STATE)
	X(STATUS_RXACT_COMMIT_FAILURE)
	X(STATUS_MAPPED_FILE_SIZE_ZERO)
	X(STATUS_TOO_MANY_OPENED_FILES)
	X(STATUS_CANCELLED)
	X(STATUS_CANNOT_DELETE)
	X(STATUS_INVALID_COMPUTER_NAME)
	X(STATUS_FILE_DELETED)
	X(STATUS_SPECIAL_ACCOUNT)
	X(STATUS_SPECIAL_GROUP)
	X(STATUS_SPECIAL_USER)
	X(STATUS_MEMBERS_PRIMARY_GROUP)
	X(STATUS_FILE_CLOSED)
	X(STATUS_TOO_MANY_THREADS)
	X(STATUS_THREAD_NOT_IN_PROCESS)
	X(STATUS_TOKEN_ALREADY_IN_USE)
	X(STATUS_PAGEFILE_QUOTA_EXCEEDED)
	X(STATUS_COMMITMENT_LIMIT)
	X(STATUS_INVALID_IMAGE_LE_FORMAT)
	X(STATUS_INVALID_IMAGE_NOT_MZ)
	X(STATUS_INVALID_IMAGE_PROTECT)
	X(STATUS_INVALID_IMAGE_WIN_16)
	X(STATUS_LOGON_SERVER_CONFLICT)
	X(STATUS_TIME_DIFFERENCE_AT_DC)
	X(STATUS_SYNCHRONIZATION_REQUIRED)
	X(STATUS_DLL_NOT_FOUND)
	X(STATUS_OPEN_FAILED)
	X(STATUS_IO_PRIVILEGE_FAILED)
	X(STATUS_ORDINAL_NOT_FOUND)
	X(STATUS_ENTRYPOINT_NOT_FOUND)
	X(STATUS_CONTROL_C_EXIT)
	X(STATUS_LOCAL_DISCONNECT)
	X(STATUS_REMOTE_DISCONNECT)
	X(STATUS_REMOTE_RESOURCES)
	X(STATUS_LINK_FAILED)
	X(STATUS_LINK_TIMEOUT)
	X(STATUS_INVALID_CONNECTION)
	X(STATUS_INVALID_ADDRESS)
	X(STATUS_DLL_INIT_FAILED)
	X(STATUS_MISSING_SYSTEMFILE)
	X(STATUS_UNHANDLED_EXCEPTION)
	X(STATUS_APP_INIT_FAILURE)
	X(STATUS_PAGEFILE_CREATE_FAILED)
	X(STATUS_NO_PAGEFILE)
	X(STATUS_INVALID_LEVEL)
	X(STATUS_WRONG_PASSWORD_CORE)
	X(STATUS_ILLEGAL_FLOAT_CONTEXT)
	X(STATUS_PIPE_BROKEN)
	X(STATUS_REGISTRY_CORRUPT)
	X(STATUS_REGISTRY_IO_FAILED)
	X(STATUS_NO_EVENT_PAIR)
	X(STATUS_UNRECOGNIZED_VOLUME)
	X(STATUS_SERIAL_NO_DEVICE_INITED)
	X(STATUS_NO_SUCH_ALIAS)
	X(STATUS_MEMBER_NOT_IN_ALIAS)
	X(STATUS_MEMBER_IN_ALIAS)
	X(STATUS_ALIAS_EXISTS)
	X(STATUS_LOGON_NOT_GRANTED)
	X(STATUS_TOO_MANY_SECRETS)
	X(STATUS_SECRET_TOO_LONG)
	X(STATUS_INTERNAL_DB_ERROR)
	X(STATUS_FULLSCREEN_MODE)
	X(STATUS_TOO_MANY_CONTEXT_IDS)
	X(STATUS_LOGON_TYPE_NOT_GRANTED)
	X(STATUS_NOT_REGISTRY_FILE)
	X(STATUS_NT_CROSS_ENCRYPTION_REQUIRED)
	X(STATUS_DOMAIN_CTRLR_CONFIG_ERROR)
	X(STATUS_FT_MISSING_MEMBER)
	X(STATUS_ILL_FORMED_SERVICE_ENTRY)
	X(STATUS_ILLEGAL_CHARACTER)
	X(STATUS_UNMAPPABLE_CHARACTER)
	X(STATUS_UNDEFINED_CHARACTER)
	X(STATUS_FLOPPY_VOLUME)
	X(STATUS_FLOPPY_ID_MARK_NOT_FOUND)
	X(STATUS_FLOPPY_WRONG_CYLINDER)
	X(STATUS_FLOPPY_UNKNOWN_ERROR)
	X(STATUS_FLOPPY_BAD_REGISTERS)
	X(STATUS_DISK_RECALIBRATE_FAILED)
	X(STATUS_DISK_OPERATION_FAILED)
	X(STATUS_DISK_RESET_FAILED)
	X(STATUS_SHARED_IRQ_BUSY)
	X(STATUS_FT_ORPHANING)
	X(STATUS_PARTITION_FAILURE)
	X(STATUS_INVALID_BLOCK_LENGTH)
	X(STATUS_DEVICE_NOT_PARTITIONED)
	X(STATUS_UNABLE_TO_LOCK_MEDIA)
	X(STATUS_UNABLE_TO_UNLOAD_MEDIA)
	X(STATUS_EOM_OVERFLOW)
	X(STATUS_NO_MEDIA)
	X(STATUS_NO_SUCH_MEMBER)
	X(STATUS_INVALID_MEMBER)
	X(STATUS_KEY_DELETED)
	X(STATUS_NO_LOG_SPACE)
	X(STATUS_TOO_MANY_SIDS)
	X(STATUS_LM_CROSS_ENCRYPTION_REQUIRED)
	X(STATUS_KEY_HAS_CHILDREN)
	X(STATUS_CHILD_MUST_BE_VOLATILE)
	X(STATUS_DEVICE_CONFIGURATION_ERROR)
	X(STATUS_DRIVER_INTERNAL_ERROR)
	X(STATUS_INVALID_DEVICE_STATE)
	X(STATUS_IO_DEVICE_ERROR)
	X(STATUS_DEVICE_PROTOCOL_ERROR)
	X(STATUS_BACKUP_CONTROLLER)
	X(STATUS_LOG_FILE_FULL)
	X(STATUS_TOO_LATE)
	X(STATUS_NO_TRUST_LSA_SECRET)
	X(STATUS_NO_TRUST_SAM_ACCOUNT)
	X(STATUS_TRUSTED_DOMAIN_FAILURE)
	X(STATUS_TRUSTED_RELATIONSHIP_FAILURE)
	X(STATUS_EVENTLOG_FILE_CORRUPT)
	X(STATUS_EVENTLOG_CANT_START)
	X(STATUS_TRUST_FAILURE)
	X(STATUS_MUTANT_LIMIT_EXCEEDED)
	X(STATUS_NETLOGON_NOT_STARTED)
	X(STATUS_ACCOUNT_EXPIRED)
	X(STATUS_POSSIBLE_DEADLOCK)
	X(STATUS_NETWORK_CREDENTIAL_CONFLICT)
	X(STATUS_REMOTE_SESSION_LIMIT)
	X(STATUS_EVENTLOG_FILE_CHANGED)
	X(STATUS_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT)
	X(STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT)
	X(STATUS_NOLOGON_SERVER_TRUST_ACCOUNT)
	X(STATUS_DOMAIN_TRUST_INCONSISTENT)
	X(STATUS_FS_DRIVER_REQUIRED)
	X(STATUS_NO_USER_SESSION_KEY)
	X(STATUS_USER_SESSION_DELETED)
	X(STATUS_RESOURCE_LANG_NOT_FOUND)
	X(STATUS_INSUFF_SERVER_RESOURCES)
	X(STATUS_INVALID_BUFFER_SIZE)
	X(STATUS_INVALID_ADDRESS_COMPONENT)
	X(STATUS_INVALID_ADDRESS_WILDCARD)
	X(STATUS_TOO_MANY_ADDRESSES)
	X(STATUS_ADDRESS_ALREADY_EXISTS)
	X(STATUS_ADDRESS_CLOSED)
	X(STATUS_CONNECTION_DISCONNECTED)
	X(STATUS_CONNECTION_RESET)
	X(STATUS_TOO_MANY_NODES)
	X(STATUS_TRANSACTION_ABORTED)
	X(STATUS_TRANSACTION_TIMED_OUT)
	X(STATUS_TRANSACTION_NO_RELEASE)
	X(STATUS_TRANSACTION_NO_MATCH)
	X(STATUS_TRANSACTION_RESPONDED)
	X(STATUS_TRANSACTION_INVALID_ID)
	X(STATUS_TRANSACTION_INVALID_TYPE)
	X(STATUS_NOT_SERVER_SESSION)
	X(STATUS_NOT_CLIENT_SESSION)
	X(STATUS_CANNOT_LOAD_REGISTRY_FILE)
	X(STATUS_DEBUG_ATTACH_FAILED)
	X(STATUS_SYSTEM_PROCESS_TERMINATED)
	X(STATUS_DATA_NOT_ACCEPTED)
	X(STATUS_NO_BROWSER_SERVERS_FOUND)
	X(STATUS_VDM_HARD_ERROR)
	X(STATUS_DRIVER_CANCEL_TIMEOUT)
	X(STATUS_REPLY_MESSAGE_MISMATCH)
	X(STATUS_MAPPED_ALIGNMENT)
	X(STATUS_IMAGE_CHECKSUM_MISMATCH)
	X(STATUS_LOST_WRITEBEHIND_DATA)
	X(STATUS_CLIENT_SERVER_PARAMETERS_INVALID)
	X(STATUS_PASSWORD_MUST_CHANGE)
	X(STATUS_NOT_FOUND)
	X(STATUS_NOT_TINY_STREAM)
	X(STATUS_RECOVERY_FAILURE)
	X(STATUS_STACK_OVERFLOW_READ)
	X(STATUS_FAIL_CHECK)
	X(STATUS_DUPLICATE_OBJECTID)
	X(STATUS_OBJECTID_EXISTS)
	X(STATUS_CONVERT_TO_LARGE)
	X(STATUS_RETRY)
	X(STATUS_FOUND_OUT_OF_SCOPE)
	X(STATUS_ALLOCATE_BUCKET)
	X(STATUS_PROPSET_NOT_FOUND)
	X(STATUS_MARSHALL_OVERFLOW)
	X(STATUS_INVALID_VARIANT)
	X(STATUS_DOMAIN_CONTROLLER_NOT_FOUND)
	X(STATUS_ACCOUNT_LOCKED_OUT)
	X(STATUS_HANDLE_NOT_CLOSABLE)
	X(STATUS_CONNECTION_REFUSED)
	X(STATUS_GRACEFUL_DISCONNECT)
	X(STATUS_ADDRESS_ALREADY_ASSOCIATED)
	X(STATUS_ADDRESS_NOT_ASSOCIATED)
	X(STATUS_CONNECTION_INVALID)
	X(STATUS_CONNECTION_ACTIVE)
	X(STATUS_NETWORK_UNREACHABLE)
	X(STATUS_HOST_UNREACHABLE)
	X(STATUS_PROTOCOL_UNREACHABLE)
	X(STATUS_PORT_UNREACHABLE)
	X(STATUS_REQUEST_ABORTED)
	X(STATUS_CONNECTION_ABORTED)
	X(STATUS_BAD_COMPRESSION_BUFFER)
	X(STATUS_USER_MAPPED_FILE)
	X(STATUS_AUDIT_FAILED)
	X(STATUS_TIMER_RESOLUTION_NOT_SET)
	X(STATUS_CONNECTION_COUNT_LIMIT)
	X(STATUS_LOGIN_TIME_RESTRICTION)
	X(STATUS_LOGIN_WKSTA_RESTRICTION)
	X(STATUS_IMAGE_MP_UP_MISMATCH)
	X(STATUS_INSUFFICIENT_LOGON_INFO)
	X(STATUS_BAD_DLL_ENTRYPOINT)
	X(STATUS_BAD_SERVICE_ENTRYPOINT)
	X(STATUS_LPC_REPLY_LOST)
	X(STATUS_IP_ADDRESS_CONFLICT1)
	X(STATUS_IP_ADDRESS_CONFLICT2)
	X(STATUS_REGISTRY_QUOTA_LIMIT)
	X(STATUS_PATH_NOT_COVERED)
	X(STATUS_NO_CALLBACK_ACTIVE)
	X(STATUS_LICENSE_QUOTA_EXCEEDED)
	X(STATUS_PWD_TOO_SHORT)
	X(STATUS_PWD_TOO_RECENT)
	X(STATUS_PWD_HISTORY_CONFLICT)
	X(STATUS_PLUGPLAY_NO_DEVICE)
	X(STATUS_UNSUPPORTED_COMPRESSION)
	X(STATUS_INVALID_HW_PROFILE)
	X(STATUS_INVALID_PLUGPLAY_DEVICE_PATH)
	X(STATUS_DRIVER_ORDINAL_NOT_FOUND)
	X(STATUS_DRIVER_ENTRYPOINT_NOT_FOUND)
	X(STATUS_RESOURCE_NOT_OWNED)
	X(STATUS_TOO_MANY_LINKS)
	X(STATUS_QUOTA_LIST_INCONSISTENT)
	X(STATUS_FILE_IS_OFFLINE)
	X(NT_STATUS_NO_SUCH_JOB)
#undef X
	{ NULL, 0 }
};

static const char *find_nterr(NTSTATUS nt_code)
{
	size_t idx = 0;

	while (nt_errs[idx].nt_errstr)
	{
		if (nt_errs[idx].nt_errcode == nt_code)
		{
			return nt_errs[idx].nt_errstr;
		}
		idx++;
	}
	return NULL;
}


/*****************************************************************************
 returns an NT error message.  not amazingly helpful, but better than a number.
 *****************************************************************************/
 BOOL get_safe_nt_error_msg(uint32 nt_code, char *msg, size_t len)
{
	const char *found_msg;

	if ((nt_code & 0xffff0000) == 0xc0070000)
	{
		WERROR werr = W_ERROR(nt_code & 0xffff);
		slprintf(msg, len-1, "Win32 code %u", werr);
		nt_code = dos_to_ntstatus(ERRDOS, werr);
		if (nt_code == NT_STATUS_UNSUCCESSFUL && werr != ERROR_GEN_FAILURE)
			/* there is no exact NT error match */
			return False;
	}
	else
	{
		slprintf(msg, len-1, "NT code %08x", nt_code);
	}

	found_msg = find_nterr(nt_code);
	if (found_msg)
	{
		safe_strcpy(msg, found_msg, len);
		return True;
	}
	return False;
}

/*****************************************************************************
 returns an NT error message.  not amazingly helpful, but better than a number.
 *****************************************************************************/
 const char *get_nt_error_msg(uint32 nt_code)
{
	char *msg = tss_malloc(FSTRING_LEN);
	get_safe_nt_error_msg(nt_code, msg, FSTRING_LEN);
	return msg;
}
