/* winbind_auth module

   based on pam_winbind ( Copyright Andrew Tridgell <tridge@samba.org> 2000 )

   largely based on pam_userdb by Christian Gafton <gafton@redhat.com> 
*/

#include "winbind_auth.h"
#include <stdarg.h>

/* prototypes from common.c */
void init_request(struct winbindd_request *req,int rq_type);
int write_sock(void *buffer, int count);
int read_reply(struct winbindd_response *response);

void *g__new(size_t type, size_t count)
{
	return malloc(type * count);
}

static int winbind_request(enum winbindd_cmd req_type,
                           struct winbindd_request *request,
                           struct winbindd_response *response)
{
	/* Fill in request and send down pipe */
	init_request(request, req_type);
	
	if (write_sock(request, sizeof(*request)) == -1) {
		return -2;
	}
	
	/* Wait for reply */
	if (read_reply(response) == -1) {
		return -2;
	}

	/* Copy reply data from socket */
	if (response->result != WINBINDD_OK) {
		return 1;
	}
	
	return 0;
}

/* talk to winbindd */
int winbind_auth_request(const char *user, const char *domain, const char *pass,
		size_t *len, void **blob)
{
	struct winbindd_request request;
	struct winbindd_response response;
	int status;

	ZERO_STRUCT(request);

	strncpy(request.data.ntauth.user, user, 
                sizeof(request.data.ntauth.user)-1);

	strncpy(request.data.ntauth.domain, domain, 
                sizeof(request.data.ntauth.domain)-1);

	strncpy(request.data.ntauth.pass, pass, 
                sizeof(request.data.ntauth.pass)-1);
	
        status = winbind_request(WINBINDD_NT_AUTH, &request, &response);
	if (status == 0)
	{
		*len = response.data.ntblob.len;
		*blob = response.data.ntblob.blob;
	}
	return status;

}

/* talk to winbindd */
int winbind_auth_chal_request(const char *user, const char *domain, 
			char *chal, int chal_len,
			char *lm_pw, int lm_pw_len,
			char *nt_pw, int nt_pw_len,
		size_t *len, void **blob)
{
	struct winbindd_request request;
	struct winbindd_response response;
	int status;

	ZERO_STRUCT(request);

	strncpy(request.data.ntchalauth.user, user, 
                sizeof(request.data.ntchalauth.user)-1);

	strncpy(request.data.ntchalauth.domain, domain, 
                sizeof(request.data.ntchalauth.domain)-1);

	memcpy(request.data.ntchalauth.ntpass, nt_pw, nt_pw_len);
	memcpy(request.data.ntchalauth.lmpass, lm_pw, lm_pw_len);
	memcpy(request.data.ntchalauth.chal, chal, chal_len);

	request.data.ntchalauth.chal_len = chal_len;
	request.data.ntchalauth.ntpass_len = nt_pw_len;
	request.data.ntchalauth.lmpass_len = lm_pw_len;
	
        status = winbind_request(WINBINDD_NTCHAL_AUTH, &request, &response);
	if (status == 0)
	{
		*len = response.data.ntblob.len;
		*blob = NULL;
		if (*len != 0)
		{
			*blob = malloc(*len);
			memcpy(*blob, response.data.ntblob.blob, *len);
		}
	}
	return status;

}

/*
 * Copyright (c) Andrew Tridgell <tridge@samba.org> 2000
 * Copyright (c) Tim Potter      <tpot@samba.org>   2000
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, and the entire permission notice in its entirety,
 *    including the disclaimer of warranties.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * ALTERNATIVELY, this product may be distributed under the terms of
 * the GNU Public License, in which case the provisions of the GPL are
 * required INSTEAD OF the above restrictions.  (This clause is
 * necessary due to a potential bad interaction between the GPL and
 * the restrictions contained in a BSD-style copyright.)
 *
 * THIS SOFTWARE IS PROVIDED `AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
