/* 
   Unix SMB/Netbios implementation.
   Version 3.0
   printing backend routines for smbd - using files_struct rather
   than only snum
   Copyright (C) Andrew Tridgell 1992-2000
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/* MKPROTO: smbd_proto.h		(do not remove this line) */

#include "includes.h"
#include "libsamba.h"
#include "smbd.h"
#include "vfs.h"
#include "others.h"


/***************************************************************************
open a print file and setup a fsp for it. This is a wrapper around
print_job_start().
***************************************************************************/

files_struct *print_fsp_open(connection_struct *conn, char *jobname,
			     uint32 access_mask, uint16 smbpid)
{
	int jobid;
	SMB_STRUCT_STAT sbuf;
	files_struct *fsp = file_new(conn);

	se_map_generic(&access_mask, &file_generic_mapping);

	if(!fsp)
		return NULL;

	jobid = print_job_start(SNUM(conn), jobname);
	if (jobid == -1) {
		file_free(fsp);
		return NULL;
	}

	/* setup a full fsp */
	fsp->print_jobid = jobid;
	fsp->fd = print_job_fd(jobid);
	fsp_stat(fsp, &sbuf);
	fsp->mode = sbuf.st_mode;
	fsp->inode = sbuf.st_ino;
	fsp->dev = sbuf.st_dev;
	fsp_set_open_time_basics(fsp);
	fsp->can_lock = True;
	fsp->can_read = False;
	fsp->can_write = True;
	fsp->print_file = True;
	fsp->is_directory = False;
	fsp->stat_open = False;
	fsp_set_dos_name(fsp, print_job_fname(jobid));
	fsp->wcp = NULL; 

	fsp->open_smbpid = smbpid;
	fsp->desired_access = access_mask;
	fsp_open_finally(fsp);

	return fsp;
}

/****************************************************************************
print a file - called on closing the file
****************************************************************************/
void print_fsp_end(files_struct *fsp)
{
	print_job_end(fsp->print_jobid);

	string_free(&fsp->dos_fsp_name);
}
