
/* 
 *  Unix SMB/Netbios implementation.
 *  Version 1.9.
 *  RPC Pipe client / server routines
 *  Copyright (C) Andrew Tridgell              1992-1999,
 *  Copyright (C) Luke Kenneth Casson Leighton 1996-1999,
 *  Copyright (C) Paul Ashton                  1997-1999.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* MKPROTO: rpc_parse_proto.h		(do not remove this line) */


#include "includes.h"
#include "debug.h"
#include "tng_misc.h"
#include "rpc_parse.h"


/*******************************************************************
 make_wks_q_query_info
 ********************************************************************/

BOOL make_wks_q_query_info(WKS_Q_QUERY_INFO *q_u,
				char *server, uint16 switch_value)  
{
	DEBUG(5,("make_wks_q_query_info\n"));

	make_buf_unistr2(&(q_u->uni_srv_name), &(q_u->ptr_srv_name), server);
	q_u->switch_value = switch_value;

	return True;
}

/*******************************************************************
 Reads or writes a WKS_Q_QUERY_INFO structure.
********************************************************************/

RPC_IO_DECLARE(wks_io_q_query_info, WKS_Q_QUERY_INFO, q_u)
{
	if (q_u == NULL)
		return False;

	prs_debug(ps, depth, desc, "wks_io_q_query_info");
	depth++;

	prs_align(ps);

	prs_uint32("ptr_srv_name", ps, depth, &(q_u->ptr_srv_name));
	if(!smb_io_unistr2("", &q_u->uni_srv_name, q_u->ptr_srv_name, ps, depth))
		return False;
	prs_align(ps);

	prs_uint16("switch_value", ps, depth, &(q_u->switch_value));
	prs_align(ps);

	return True;
}

/*******************************************************************
 wks_info_100
 ********************************************************************/

BOOL make_wks_info_100(WKS_INFO_100 *inf,
				uint32 platform_id, uint32 ver_major, uint32 ver_minor,
				char *my_name, char *domain_name)
{
	DEBUG(5,("WKS_INFO_100: %d\n", __LINE__));

	inf->platform_id = platform_id; /* 0x0000 01f4 - unknown */
	inf->ver_major   = ver_major;   /* os major version */
	inf->ver_minor   = ver_minor;   /* os minor version */

	make_buf_unistr2(&(inf->uni_compname), &(inf->ptr_compname), my_name    );
	make_buf_unistr2(&(inf->uni_lan_grp ), &(inf->ptr_lan_grp ), domain_name);

	return True;
}

/*******************************************************************
 Reads or writes a WKS_INFO_100 structure.
********************************************************************/

static RPC_IO_DECLARE(wks_io_wks_info_100, WKS_INFO_100, inf)
{
	if (inf == NULL)
		return False;

	prs_debug(ps, depth, desc, "wks_io_wks_info_100");
	depth++;

	prs_align(ps);

	prs_uint32("platform_id ", ps, depth, &(inf->platform_id )); /* 0x0000 01f4 - unknown */
	prs_uint32("ptr_compname", ps, depth, &(inf->ptr_compname)); /* pointer to computer name */
	prs_uint32("ptr_lan_grp ", ps, depth, &(inf->ptr_lan_grp )); /* pointer to LAN group name */
	prs_uint32("ver_major   ", ps, depth, &(inf->ver_major   )); /* 4 - major os version */
	prs_uint32("ver_minor   ", ps, depth, &(inf->ver_minor   )); /* 0 - minor os version */

	if(!smb_io_unistr2("", &inf->uni_compname, inf->ptr_compname, ps, depth))
		return False;
	prs_align(ps);

	if(!smb_io_unistr2("", &inf->uni_lan_grp, inf->ptr_lan_grp , ps, depth))
		return False;
	prs_align(ps);

	return True;
}

/*******************************************************************
 make_wks_r_query_info

 only supports info level 100 at the moment.

 ********************************************************************/

BOOL make_wks_r_query_info(WKS_R_QUERY_INFO *r_u,
				uint32 switch_value, WKS_INFO_100 *wks100,
				int status)  
{
	DEBUG(5,("make_wks_r_unknown_0: %d\n", __LINE__));

	r_u->switch_value = switch_value;  /* same as in request */

	r_u->ptr_1     = 1;          /* pointer 1 */
	r_u->wks100    = wks100;

	r_u->status    = status;

	return True;
}

/*******************************************************************
 Reads or writes a structure.
********************************************************************/

RPC_IO_DECLARE(wks_io_r_query_info, WKS_R_QUERY_INFO, r_u)
{
	if (r_u == NULL)
		return False;

	prs_debug(ps, depth, desc, "wks_io_r_query_info");
	depth++;

	prs_align(ps);

	prs_uint16("switch_value", ps, depth, &(r_u->switch_value)); /* level 100 (0x64) */
	prs_align(ps);

	prs_uint32("ptr_1       ", ps, depth, &(r_u->ptr_1       )); /* pointer 1 */
	if(!wks_io_wks_info_100("inf", r_u->wks100, ps, depth))
		return False;

	prs_uint32("status      ", ps, depth, &(r_u->status));

	return True;
}

/*******************************************************************
 Reads or writes a structure.
********************************************************************/
BOOL make_wks_q_user_enum(WKS_Q_USER_ENUM *q_u,
			  const char *server, uint16 switch_value)
{
	DEBUG(5,("make_wks_q_user_enum\n"));

	make_buf_unistr2(&(q_u->uni_srv_name), &(q_u->ptr_srv_name), server);
	q_u->unk0 = switch_value;
	q_u->unk1 = switch_value;
	q_u->unk2 = 0x0;
	q_u->unk3 = 0x0;
	q_u->unk4 = 0x0;

	q_u->unk_ptr = 0x1;
	q_u->max_size = 0xffffffff;

	return True;
}

static RPC_IO_DECLARE(wks_io_logged_in_user, WKS_LOGGED_IN_USER, q_u)
{
	if (q_u == NULL)
		return False;

	prs_set_depth(ps, depth);
	prs_debug(ps, -1, desc, "wks_io_logged_in_user");
	prs_inc_depth(ps);
	depth = prs_depth(ps);

	prs_uint32("ptr_username", ps, depth, &q_u->ptr_username);
	if (q_u->ptr_username)
	{
		if (! RPC_MARSH_SUBCALL(smb_io_unistr2_defer, q_u, uni_username))
			return False;
	}

	prs_uint32("ptr_domain", ps, depth, &q_u->ptr_domain);
	if (q_u->ptr_username)
	{
		if (! RPC_MARSH_SUBCALL(smb_io_unistr2_defer, q_u, uni_domain))
			return False;
	}

	prs_uint32("ptr_unk", ps, depth, &q_u->ptr_unk);
	if (q_u->ptr_username)
	{
		if (! RPC_MARSH_SUBCALL(smb_io_unistr2_defer, q_u, uni_unk))
			return False;
	}

	prs_uint32("ptr_logonserver", ps, depth, &q_u->ptr_logonserver);
	if (q_u->ptr_username)
	{
		if (! RPC_MARSH_SUBCALL(smb_io_unistr2_defer, q_u, uni_logonserver))
			return False;
	}

	return True;
}

RPC_IO_DECLARE(wks_io_q_user_enum, WKS_Q_USER_ENUM, q_u)
{
	if (q_u == NULL)
		return False;

	prs_debug(ps, depth, desc, "wks_io_q_user_enum");
	depth++;

	prs_align(ps);

	prs_uint32("ptr_srv_name", ps, depth, &(q_u->ptr_srv_name));
	if(!smb_io_unistr2("", &q_u->uni_srv_name, q_u->ptr_srv_name, ps, depth))
		return False;

	/* one of the following uint32 must be an info level */
	prs_align(ps);
	prs_uint32("unk0", ps, depth, &q_u->unk0);
	prs_uint32("unk1", ps, depth, &q_u->unk1);

	/* this could be an ENUM_HND */
	prs_uint32("unk_ptr", ps, depth, &q_u->unk_ptr);
	prs_uint32("unk2", ps, depth, &q_u->unk2);

	prs_uint32("unk3", ps, depth, &q_u->unk3);
	prs_uint32("max_size", ps, depth, &q_u->max_size);
	prs_uint32("unk4", ps, depth, &q_u->unk4);

	return True;
}

RPC_IO_DECLARE(wks_io_r_user_enum, WKS_R_USER_ENUM, q_u)
{
	uint32 i;

	if (q_u == NULL)
		return False;

	prs_debug(ps, depth, desc, "wks_io_r_user_enum");
	depth++;
	prs_set_depth(ps, depth);

	prs_uint32("level1", ps, depth, &q_u->level1);
	prs_uint32("level2", ps, depth, &q_u->level2);
	prs_uint32("ptr1", ps, depth, &q_u->ptr1);
	prs_uint32("count1", ps, depth, &q_u->count1);
	prs_uint32("ptr2", ps, depth, &q_u->ptr2);
	prs_uint32("count2", ps, depth, &q_u->count2);

	if (UNMARSHALLING(ps))
	{
		q_u->users = g_new(WKS_LOGGED_IN_USER, q_u->count2);
	}
	if (! q_u->users)
		return False;

	if (!prs_start_pending(ps, "user enum"))
		return False;
	for (i = 0; i < q_u->count2; i++)
	{
		if (! wks_io_logged_in_user("", &q_u->users[i], ps, depth))
			return False;
	}
	if (!prs_stop_pending(ps))
		return False;

	prs_uint32("count3", ps, depth, &q_u->count3);
	prs_uint32("unk", ps, depth, &q_u->unk);
	prs_uint32("status", ps, depth, &q_u->status);

	return True;
}


RPC_IO_DECLARE(wks_io_q_unjoin, WKS_Q_UNJOIN, q_u)
{
	if (q_u == NULL)
		return False;

	prs_debug(ps, depth, desc, "wks_io_q_unjoin");
	depth++;

	prs_align(ps);

	prs_uint32("ptr_srv_name", ps, depth, &(q_u->ptr_srv_name));
	if(!smb_io_unistr2("", &q_u->uni_srv_name, q_u->ptr_srv_name, ps, depth))
		return False;

	prs_align(ps);
	prs_uint32("unk0", ps, depth, &q_u->unk0);
	prs_uint32("unk1", ps, depth, &q_u->unk1);
	prs_uint32("unk2", ps, depth, &q_u->unk2);

	return True;
}

RPC_IO_DECLARE(wks_io_r_unjoin, WKS_R_UNJOIN, q_u)
{
	if (q_u == NULL)
		return False;

	prs_debug(ps, depth, desc, "wks_io_r_unjoin");
	depth++;

	prs_uint32("status", ps, depth, &q_u->status);

	return True;
}
