/* 
   Unix SMB/Netbios implementation.
   Version 1.9.
   connection claim routines
   Copyright (C) Andrew Tridgell 1998
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/* MKPROTO: smbd_proto.h		(do not remove this line) */

#include "includes.h"
#include <errno.h>
#include "libsamba.h"
#include "smbd.h"


extern fstring remote_machine;
static TDB_CONTEXT *tdb;


/****************************************************************************
 Return the connection tdb context (used for message send all).
****************************************************************************/

TDB_CONTEXT *conn_tdb_ctx(void)
{
	return tdb;
}


static
void make_conn_key(const connection_struct *conn, const char *name,
		   TDB_DATA *pkbuf, struct connections_key *pkey)
{
	ZERO_STRUCTP(pkey);
	pkey->pid = sys_getpid();
	pkey->cnum = conn?conn->cnum:-1;
	fstrcpy(pkey->name, name);
	pkbuf->dptr = (char *)pkey;
	pkbuf->dsize = sizeof(*pkey);
}


/****************************************************************************
 Delete a connection record.
****************************************************************************/
BOOL yield_connection(connection_struct *conn,char *name,int max_connections)
{
	struct connections_key key;
	TDB_DATA kbuf;

	if (!tdb)
		return False;

	DEBUG(3,("Yielding connection to %s\n",name));

	make_conn_key(conn, name, &kbuf, &key);

	if (tdb_delete(tdb, kbuf) != 0) {
		DEBUG(0,("yield_connection: tdb_delete for name %s failed with error %s.\n",
				name, tdb_errorstr(tdb) ));
		return (False);
	}

#ifdef WITH_UTMP
	if(conn)
		utmp_yield(key.pid, conn);
#endif

	return(True);
}


/****************************************************************************
 Claim an entry in the connections database.
****************************************************************************/

BOOL claim_connection(connection_struct *conn,char *name,int max_connections,BOOL Clear)
{
	struct connections_key key;
	struct connections_data crec;
	TDB_DATA kbuf, dbuf;
	BOOL ret = True;

	if (!tdb) {
		tdb = tdb_open_log(tmp_path ("connections.tdb"), 0,
				   TDB_CLEAR_IF_FIRST,
				   O_RDWR | O_CREAT, 0644);
	}
	if (!tdb)
		return False;

	DEBUG(5,("claiming %s %d\n",name,max_connections));

	make_conn_key(conn, name, &kbuf, &key);

	/* fill in the crec */
	ZERO_STRUCT(crec);
	crec.magic = 0x280267;
	crec.pid = sys_getpid();
	crec.cnum = conn?conn->cnum:-1;
	if (conn) {
		crec.uid = conn->uid;
		crec.gid = conn->gid;
		StrnCpy(crec.name,
			lp_servicename(SNUM(conn)),sizeof(crec.name)-1);
	}
	crec.start = time(NULL);
	
	StrnCpy(crec.machine,remote_machine,sizeof(crec.machine)-1);
	StrnCpy(crec.addr,conn?conn->client_address:client_addr(),sizeof(crec.addr)-1);

	dbuf.dptr = (char *)&crec;
	dbuf.dsize = sizeof(crec);

	if (tdb_store(tdb, kbuf, dbuf, TDB_REPLACE) != 0) {
		DEBUG(0,("claim_connection: tdb_store failed with error %s.\n",
			tdb_errorstr(tdb) ));
		ret = False;
	}

#ifdef WITH_UTMP
	if (ret && conn)
	    utmp_claim(&crec, conn);
#endif

	return ret;
}
