/* 
   Unix SMB/CIFS implementation.
   server specific string routines
   Copyright (C) Andrew Tridgell 2001
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/* MKPROTO: smbd_proto.h		(do not remove this line) */

#include "includes.h"
#include "byteorder.h"
#include "libsamba.h"
#include "libsmb.h"
#include "tng_misc.h"
#include "smbd.h"

int srvstr_push(void *base_ptr, void *dest, const char *src, int dest_len, int flags)
{
	return push_string(base_ptr, dest, src, dest_len, flags);
}

int srvstr_pull(void *base_ptr, char *dest, const void *src, int dest_len, int src_len, 
		int flags)
{
	int ret;

	/* This should really be in pull_string() */
	if (flags & STR_IS_IN_SMBBUF)
	{
		int remaining;

		if (! cifs_ptr_in_smbbuf(base_ptr, src))
			return 0;
		remaining = smb_bufrem(base_ptr, src);
		if (src_len < 0)
			src_len = remaining;
		else
			src_len = MIN(src_len, remaining);
	}

	ret = pull_string(base_ptr, dest, src, dest_len, src_len, flags);
	DEBUG(5, ("srvstr_pull(%d, %d), got [%s], returning %d\n",
		  dest_len, src_len, dest, ret));
	dump_data(9, src, src_len);
	return ret;
}

/* pull a string from the smb_buf part of a packet. In this case the
   string can either be null terminated or it can be terminated by the
   end of the smbbuf area 
*/
int srvstr_pull_buf(void *inbuf, char *dest, const void *src, int dest_len, int flags)
{
	if (! cifs_ptr_in_smbbuf(inbuf, src))
		return 0;

	return srvstr_pull(inbuf, dest, src, dest_len, smb_bufrem(inbuf, src), flags);
}


void dos_to_unix(char *str, BOOL inplace)
{
	size_t len;

	if (! str)
		return;
	if (! inplace)
		return;

	len = strlen(str) + 1;

	convert_string(CH_DOS, CH_UNIX,
		       str, len,
		       str, len);

	str[len-1] = '\0'; /* nul terminate in any case */
}


const char *mangle_from_dos_to_unix(connection_struct *conn, const char *dosname,
				    BOOL need83, BOOL cache83)
{
	size_t namelen = strlen(dosname) + 1;
	char *manglenametmp;
	const char *unixtmp;
	BOOL ret;

	/*
	 * 13 chars (8+3+dot+nul) should be enough.
	 * mangle_name_83() uses slprintf(.., 12chars) + strcat(4chars)
	 * add a factor of 2.
	 */
	namelen = (namelen + 17) * 2;

	manglenametmp = g_new(char, namelen);

	unsafe_strcpy(manglenametmp, dosname);

	ret = name_map_mangle(conn, manglenametmp, need83, cache83);

	if (! ret)
	{
		safe_free(manglenametmp);
		DEBUG(1, ("mangle_from_dos_to_unix(%s) failed, returning empty\n",
			  dosname));
		return "";
	}

	if (need83)
		strupper(manglenametmp);

	unixtmp = dos_to_unix_static(manglenametmp);
	safe_free(manglenametmp);

	return unixtmp;
}
